#!/bin/bash
#
# This script updates the database structure
#
# Updates from versions: 1.0.x, 1.1.x, 1.2.x 2.0.x
# Updates to version: 2.1.0
#
# Author: Abilis <support@antek.it>
# Date:   05/02/2013
# Ver:    2.1.0-1
#
# Changelog
# 2.1.0: Fixed an error in codes table (ItoS was 36 instead of 37)
# 2.0: Added Connection types from 0x1D to 0x28
#

# --- CONSTANTS DEFINITIONS ---

# Script version
VER="2.1.0-1"
DBVER="5"

# The full path of configuration file
CONFFILE="/etc/opt/abilis/collector.conf"

# Get full path of the update script
SCRIPT_FULL_PATH=`readlink -f $0`
SCRIPT_DIR_PATH=`dirname ${SCRIPT_FULL_PATH}`

# SQL file for updating db structure
STRCT_UPD_1_TO_2="${SCRIPT_DIR_PATH}/sql/global_1_to_2.sql"
STRCT_UPD_2_TO_3="${SCRIPT_DIR_PATH}/sql/global_2_to_3.sql"
STRCT_UPD_3_TO_4="${SCRIPT_DIR_PATH}/sql/global_3_to_4.sql"
STRCT_UPD_4_TO_5="${SCRIPT_DIR_PATH}/sql/global_4_to_5.sql"

# SQL template file for updating collector tables 
TBL_TEMPL_UPD_1_TO_2="${SCRIPT_DIR_PATH}/sql/collector_table_1_to_2.sql.tpl"
TBL_TEMPL_UPD_2_TO_3="${SCRIPT_DIR_PATH}/sql/collector_table_2_to_3.sql.tpl"

# --- END CONSTANTS DEFINITIONS ---

echo
echo "#################################"
echo "#   Collector database update   #"
echo "#          Ver. ${VER}           #"
echo "#              ***              #"
echo "#    Abilis: www.abilis.net     #"
echo "#################################"
echo

# Check if the install script is running as root
if [[ ${UID} != "0" ]]; then
  echo -e "ERROR: You need to be root to install the collector database\n"
  exit 1
fi

# Check if CONFFILE exists
if [[ ! -e ${CONFFILE} ]]; then
  echo -e "\nERROR: File ${CONFFILE} not found\n"
  echo "If collector was installed into a different place please specify the"
  echo "collector.conf path into the CONFFILE variable inside this script."
  echo
  exit 1
fi

# Obtain database info from the CONFFILE
MYSQLHOST=`grep MYSQLHOST ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPORT=`grep MYSQLPORT ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLDB=`grep MYSQLDB ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLUSER=`grep MYSQLUSER ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPASS=`grep MYSQLPASS ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLTABLES=`egrep ^MYSQLTABLE ${CONFFILE} | sort | uniq | awk -F ':' '{print $2}'`

# Check if required database fields are present
for keyword in MYSQLHOST MYSQLDB MYSQLUSER; do
  if [[ -z ${!keyword} ]]; then
    echo -e "\nERROR: ${keyword} is not specified in ${CONFFILE}!"
    echo -e "Please set the correct db credentials in ${CONFFILE}\n"
    exit 2
  fi
done

# Set default database port if not set into collector.conf
if [[ -z ${MYSQLPORT} ]]; then
  echo "INFO: MYSQLPORT is not set, assuming 3306"
  MYSQLPORT=3306
fi

# Assume no password if MYSQLPASS is not set
if [[ -z ${MYSQLPASS} ]]; then
  echo "INFO: MYSQLPASS is not set"
else
  MYSQLPASS="-p${MYSQLPASS}"
fi

# Ask the user if he wants to perform a database backup before update
choice=""
do_backup=0
while [[ ! ${choice} =~ ^[yn]$ ]]; do
  echo -e "\nDatabase backup is STRONGLY SUGGESTED!"
  echo -en "Do you want to backup your current database (y/n)? "
  read choice

  if [[ ${choice} == "y" ]]; then 
    echo -e "\nOk, DB backup will be made, a wisdom choice man!"

    backup_path_ok=0
    while [[ ${backup_path_ok} -eq 0 ]]; do
      echo -e "\nPlease choose the path where you want to save the backup"
      echo -en "(press enter for default: ${SCRIPT_DIR_PATH}): "
      read backup_path
      
      if [[ -z ${backup_path} ]]; then
        backup_path=${SCRIPT_DIR_PATH}
      fi

      if [[ -d ${backup_path} ]]; then
        backup_path_ok=1
        do_backup=1
      else
        echo -e "\nThe path you specified is not valid. Please enter a valid one"
      fi
    done

  elif [[ ${choice} == "n" ]]; then 
    echo -e "\nOk, DB backup will NOT be made. :/"
  else 
    echo -e "Please choose either y or n" 
  fi

done

# Summary
echo
echo "###############"
echo "#   Summary   #"
echo "###############"
echo
echo -e "MySQL Host:\t${MYSQLHOST}"
echo -e "MySQL Port:\t${MYSQLPORT}"
echo -e "MySQL DB:\t${MYSQLDB}"
echo -e "MySQL User:\t${MYSQLUSER}"
echo
if [[ ${do_backup} == 1 ]]; then
  echo "DB Backup: YES"
  echo "DB Backup file: ${backup_path}/${MYSQLDB}.sql.gz"
else
  echo "DB Bckup: NO"
fi
echo

choice=""
while [[ ! ${choice} =~ (^yes$)|(^no$) ]]; do  
  echo -n "Do you want to continue (yes/no)? " 
  read choice 
  echo 

  if [[ ${choice} == "yes" ]]; then 
    echo -e "*** Beginning update ***\n" 
  elif [[ ${choice} == "no" ]]; then 
    echo -e "*** Update aborted! ***\n" 
    exit 1 
  else 
    echo -e "Please choose either yes or no" 
  fi 
done

MYSQL_CMD="mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS}"
MYSQLDUMP_CMD="mysqldump ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS}"

# Try to determine the version. if table version_info doesn't exists in
# current database then it is a pre 1.1.x version
#
#db_ver=1 is the version assigned for collectors 1.0.x

current_db_ver="1"

${MYSQL_CMD} -e 'DESCRIBE version_info' > /dev/null 2>&1
if [[ $? == 0 ]]; then
  
  # The version_info table exists, this is a version > than 1.0.x
  # Now get the current database version
  current_db_ver=`mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS} -e 'SELECT version from version_info LIMIT 0,1' -E | grep version | awk '{print $2}'`
  
  if [[ ${current_db_ver} == ${DBVER} ]]; then
    echo "INFO: Current DB is already updated, nothing to do"
    echo
    exit 0
  fi

  echo "INFO: Current DB version is ${current_db_ver}"

else # table 'version_info' not found
  echo "INFO: table 'version_info' NOT found. Current database is from a"
  echo "      1.0.x version"
fi

# (eventually) backup current db
if [[ ${do_backup} == 1 ]]; then
  echo -n "Executing database backup... "
  ${MYSQLDUMP_CMD} | gzip -c > ${backup_path}/${MYSQLDB}.sql.gz
  
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Backup failed, aborting :(\n"
    exit 255
  fi

fi

# BEGIN updating database structure

# If current_db_ver is 1 then we need to upgrade to version 2
if [[ ${current_db_ver} == "1" ]]; then

  echo
  echo -n "Updating DB global structure from 1 to 2... "
  ${MYSQL_CMD} < ${STRCT_UPD_1_TO_2}
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Update failed, sorry :(\n"
  exit 255
  fi
  
  echo "Updating collector tables from 1 to 2:"
  for table in ${MYSQLTABLES}; do
    echo -en "  Updating table \"$table\"... "
    sed "s/__TABLE_NAME__/${table}/g" ${TBL_TEMPL_UPD_1_TO_2} | ${MYSQL_CMD} 
    if [[ $? -eq 0 ]]; then
      echo "Done!"
    else
      echo -e "\nERROR: Update failed, sorry :(\n"
      exit 255
    fi
  done
  echo "Done!"
  
  # Now DB is updated to version 2
  current_db_ver="2"
fi

if [[ ${current_db_ver} == "2" ]]; then

  # Now upgrade DB structure to 3
  echo
  echo -n "Updating global structure from 2 to 3... "
  ${MYSQL_CMD} < ${STRCT_UPD_2_TO_3}
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Update failed, sorry :(\n"
    exit 255
  fi

  echo "Updating collector tables from 2 to 3:"
  for table in ${MYSQLTABLES}; do
    echo -en "  Updating table \"$table\"... "
    sed "s/__TABLE_NAME__/${table}/g" ${TBL_TEMPL_UPD_2_TO_3} | ${MYSQL_CMD} 
    if [[ $? -eq 0 ]]; then
      echo "Done!"
    else
      echo -e "\nERROR: Update failed, sorry :(\n"
      exit 255
    fi
  done

  current_db_ver="3"
fi

if [[ ${current_db_ver} == "3" ]]; then

  # Now upgrade DB structure to 4
  echo
  echo -n "Updating global structure from 3 to 4... "
  ${MYSQL_CMD} < ${STRCT_UPD_3_TO_4}
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Update failed, sorry :(\n"
    exit 255
  fi

  # No collector table upgrade is needed for version 4

  current_db_ver="4"
fi

if [[ ${current_db_ver} == "4" ]]; then

  # Now upgrade DB structure to 5
  echo
  echo -n "Updating global structure from 4 to 5... "
  ${MYSQL_CMD} < ${STRCT_UPD_4_TO_5}
  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Update failed, sorry :(\n"
    exit 255
  fi

  # No collector table upgrade is needed for version 5

  current_db_ver="5"
fi

# END updating

echo -e "\nDB Update to version ${current_db_ver} succesfully completed\n"

