#!/bin/bash
#
# Abilis Collector Upgrader
#
# Author: Dario Fiumicello <fiumicello@antek.it>
# Ver: 2.1.0-1
#
# Copyright (c) 2013 - Abilis


# Do not change those constants since they are coded also into collector code
# If you need to change them please recompile the collector
INSTALL_DIR=/opt/abilis
BIN_DIR=${INSTALL_DIR}/bin
ETC_DIR=/etc/opt/abilis
LOG_DIR=/var/opt/abilis/log/collector
RUN_DIR=/var/opt/abilis/run
SHARE_DIR=/opt/abilis/share/collector
VERSION=2.1.0

INST_PROGRAM_FILE=collector
INST_CONF_FILE=collector.conf
INST_INIT_FILE=collector_init
INST_UNINST_FILE=uninstall.sh
INST_DB_DIR=database
README_FILE=README
VERSION_FILE=VERSION

BASH=${BASH:=`which bash`} || (echo "FATAL: bash not found!"; exit 1)
SED=`which sed` || (echo "FATAL: sed not found!"; exit 1)
DIRNAME=`which dirname` || (echo "FATAL: dirname not found!"; exit 1)
RM=`which rm` || (echo "FATAL: rm not found!"; exit 1)
PIDOF=`which pidof` || (echo "FATAL: pidof not found!"; exit 1)
STTY=`which stty` || (echo "FATAL: stty not found!"; exit 1)

### FUNCTIONS DEFINITIONS ###

#
# function ask_user
#
# Ask a question to the user and force him to choose between a set of possible
# answers.
#
# Params:
# 1) The question to prompt to the user
# 2) A string with possible answers separated by a space
# 3) A variable which will contain the answer
#
# Returns always 0
#
function ask_user {

  if [[ $# != 3 ]]; then
    echo "ask_user: FATAL! 3 argouments are needed"
    exit 255
  fi

  local question=${1}
  local answers=${2}
  local result=${3}

  regex=`echo ${answers} | ${SED} 's/ /|/g'`

  local user_answer
  while [[ true ]]; do
    echo -en "${question} "
    read user_answer
    if [[ "${user_answer}" =~ ${regex} ]]; then
      eval $result="'${user_answer}'";
      return 0
    else
      echo "Please choose among \"$answers\""
    fi
  done
}

#
# func check_return_code
#
# Check if a program exitcode is equal or different than 0. If it is equal
# prints "[OK]\n", otherwise prints "[FAIL]\n" followed by an error message
# specified by the user
#
# Params:
# 1) The exitcode to check
# 2) (optional) The message to print in case of error
#
# Returns the passed exitcode
#
function check_return_code {
  if [[ $# -gt 2 ]]; then
    echo "check_return_code: FATAL! no more than 2 argouments are accepted"
    exit 255
  fi

  local code=${1}
  local errmsg=${2}

  if [[ ${code} == "0" ]]; then
    echo "[OK]"
  else
    echo -e "\n$errmsg"
  fi

  return ${code}
}

### END FUNCTIONS DEFINITIONS ###



### MAIN PROGRAM ###

echo
echo "################################"
echo "#    Abilis Linux Collector    #"
echo "#           Upgrader           #"
echo "#             ---              #"
echo "#    (c) 2013 - abilis.net     #"
echo "################################"
echo

# Check if the install script is running as root
if [[ ${UID} != "0" ]]; then
  echo -e "ERROR: You need to be root to install the collector\n"
  exit 1
fi

# Check if a collector is still running
${PIDOF} collector > /dev/null
if [[ $? == "0" ]]; then
  echo "ERROR: A collector is already running!"
  echo -e "Please stop it before installing the new collector.\n"
  exit 1
fi

# Check if a collector is already present in ${BIN_DIR}
if [[ ! -x "${BIN_DIR}/collector" ]]; then
  echo "ERROR: Unable to find a collector in ${BIN_DIR}!"
  echo -e "\nPlease note that it is possible to upgrade"
  echo -e "ONLY FROM VERSIONS >= 2.0.0."
  echo -e "\nIf you have an older version please run the install.sh script.\n"
  exit 1
fi

# Collector was found. Checking file ${SHARE_DIR}/${VERSION_FILE}. If file 
# doesn't exist then the collector version is assumed to be 2.0.0
CURR_VERSION="2.0.0"
if [[ -e "${SHARE_DIR}/${VERSION_FILE}" ]]; then
  CURR_VERSION=`cat ${SHARE_DIR}/${VERSION_FILE}`
fi

# Summary
echo "Installed Version: ${CURR_VERSION}"
echo "New Version:       ${VERSION}"
echo

if [[ ${CURR_VERSION} == ${VERSION} ]]; then
  echo -e "Nothing to upgrade, I quit!\n"
  exit 0
fi

ask_user \
  "Do you want to begin upgrade? (y/n)" \
  "y n" \
  confirm_upgrade

if [[ ${confirm_upgrade} == "n" ]]; then
  echo -e "Upgrade aborted, I quit!\n"
  exit 0
fi

# BEGIN UPGRADE

# 2.0.0 => 2.0.1
# - New collector binary
# - Viewer is now present in ${SHARE_DIR}/viewer
# - File ${SHARE_DIR}/${VERSION_FILE} is created
if [[ ${CURR_VERSION} == "2.0.0" ]]; then
  echo -en "\nUpgrading from 2.0.0 to 2.0.1... "
  cp collector ${BIN_DIR}/ && \
  chmod 750 ${BIN_DIR}/collector && \
  cp -ax viewer ${SHARE_DIR} && \
  chmod -R 755 ${SHARE_DIR}/viewer && \
  cp uninstall.sh ${SHARE_DIR}/ && \
  chmod 750 ${SHARE_DIR}/uninstall.sh && \
  cp README ${SHARE_DIR}/ && \
  chmod 644 ${SHARE_DIR}/README && \
  echo "2.0.1" > ${SHARE_DIR}/${VERSION_FILE}
  check_return_code \
    $? \
    "Error while upgrading from 2.0.0 to 2.0.1. I quit!" || \
    exit $?

  CURR_VERSION="2.0.1"
fi

# 2.0.1 => 2.1.0
# - New collector binary
# - Fixed database typo in 'codes' table
if [[ ${CURR_VERSION} == "2.0.1" ]]; then
  echo -en "\nUpgrading from 2.0.1 to 2.1.0... "
  cp collector ${BIN_DIR}/ && \
  chmod 750 ${BIN_DIR}/collector && \
  cp uninstall.sh ${SHARE_DIR}/ && \
  chmod 750 ${SHARE_DIR}/uninstall.sh && \
  cp README ${SHARE_DIR}/ && \
  chmod 644 ${SHARE_DIR}/README && \
  cp -ax database ${SHARE_DIR}/ && \
  chmod +x ${SHARE_DIR}/*.sh && \
  echo "2.1.0" > ${SHARE_DIR}/${VERSION_FILE} && \
  check_return_code \
    $? \
    "Error while upgrading from 2.0.1 to 2.1.0. I quit!" || \
    exit $?

  CURR_VERSION="2.1.0"
fi

${STTY} -echo
echo -e "\nSoftware upgrade complete!";
echo -e "Press a key to continue with database upgrade"
read dummy
${STTY} echo

${SHARE_DIR}/database/update.sh

# END UPGRADE

echo -e "\n### All done! ###"
echo -e "\nCollector is now upgraded to version ${CURR_VERSION}\n"

